/*
 * SecureServerToolkit.java
 */

package edu.stanford.peer.secrmi;

import java.io.*;
import java.rmi.server.RMIServerSocketFactory;
import java.security.*;
import java.security.cert.CertificateException;
import javax.net.ssl.SSLServerSocketFactory;
import com.sun.net.ssl.*;

/**
 * The SSLRMIServerSocketFactory class 
 * implements RMIServerSocketFactory and produces SSL server sockets.  
 *
 * @author  Ninghui Li
 */
public class SSLRMIServerSocketFactory 
        implements RMIServerSocketFactory
{
    transient private SSLServerSocketFactory  actualFactory;

	private KeyStore ks;
	private char[]   pw;

    /**
	 * Creates a SSLRMIServerSocketFactory with the given KeyStore and password.
	 * @param keyStore  the key store 
	 * @param password  the password to access the private key in the key store
	 */
    public SSLRMIServerSocketFactory(KeyStore keyStore, char[] password) 
	    throws NoSuchProviderException, NoSuchAlgorithmException, 
    		   KeyStoreException, CertificateException, UnrecoverableKeyException, 
    		   KeyManagementException
    {
		ks = keyStore;
		pw = password;
		initiate();
    }

    /**
     * Create a SSL server socket.
     * @param port  port number
     * @returns the SSL Server socket just created
     */
    public java.net.ServerSocket createServerSocket(int port) throws IOException
    {
        if (actualFactory == null) {
        	try {
        		initiate();
        	} catch (Exception e) {
        		throw new IOException("SSL Failure");
        	}
        }
        return actualFactory.createServerSocket(port);
    }

    private void initiate() 
    	throws NoSuchProviderException, NoSuchAlgorithmException, 
    		   KeyStoreException, CertificateException, 
			   UnrecoverableKeyException, KeyManagementException
    {
    	KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509", "SunJSSE");
    	kmf.init(ks, pw);

    	// Create SSL Context using default trust managers and default secure random
    	com.sun.net.ssl.SSLContext sslContext = SSLContext.getInstance("TLS");
    	sslContext.init(kmf.getKeyManagers(), null, null);

    	actualFactory = sslContext.getServerSocketFactory();
    }
}