/*
 * SignedRemoteObject.java
 */

package edu.stanford.peer.secrmi;

import java.io.Serializable;
import java.security.SignedObject;
import java.security.PrivateKey;
import java.security.Signature;

/**
 * The SignedRemoteObject class stores a SignedObject object and implements
 * Remote, so that it can be stored in RMI registry.  It exists only because
 * SignedObject doesn't implement Remote.
 *
 * @version 1.01 12/31/01
 * @author  Ninghui Li
 */
public class SignedRemoteObject implements java.rmi.Remote, Serializable
{
	private SignedObject	myobject;

    /**
	 * Create a SignedRemoteObject by giving an object and a signing tool.
	 * 
	 * @param object        the object to be sigend
	 * @param signingKey    the key to be used
	 * @param signingEngine the signing engine to be used
	 */
	public SignedRemoteObject(Serializable object, 
							  PrivateKey signingKey,
                    		  Signature signingEngine)
				throws java.io.IOException, java.security.InvalidKeyException,
				       java.security.SignatureException
	{
		this(new SignedObject(object, signingKey, signingEngine));
	}

	/**
	 * Create a SignedRemoteObject by giving an SignedObject.
	 * 
	 * @param signedObj   the SignedObject to be stored
	 */
	public SignedRemoteObject(SignedObject signedObj) 
	{
		myobject = signedObj;
	}

	/**
	 * Get the stored SignedObject
	 * 
	 * @returns the stored SigendObject
	 */
	public SignedObject getSignedObject()
	{
		return myobject;
	}
}